#!/usr/bin/env python
# encoding: utf-8
"""
check_prerequisites.py

Created by Rod Docking on 2011-04-18.
Copyright (c) 2011 Canada's Michael Smith Genome Sciences Centre. All rights reserved.
"""

import sys
import os
import subprocess

def check_prerequisites():
    """
    Check whether all requirements are installed and are up-to-date
    """
    
    # Check for Python version (though it will provably fail well before here if it's wrong)
    assert(sys.version_info[0] == 2)
    assert(sys.version_info[1] in [6,7])
            
    # Check that ABYSS version is correct
    args = ['ABYSS', '--version']
    proc = subprocess.Popen(args, shell=False, cwd=os.getcwd(), stdout=subprocess.PIPE)
    version_string = proc.stdout.readlines()[0].rstrip()
    assert(version_string == "ABYSS (ABySS) 1.2.7")

    # TODO - give DAssembler a proper version string
    args = ['DAssembler', '--help']
    proc = subprocess.Popen(args, shell=False, cwd=os.getcwd(), stdout=subprocess.PIPE)
    version_string = proc.stdout.readlines()[0].rstrip()
    assert(version_string == "Usage: DAssembler [OPTION]... [READS]")
    
    # Check that samtools is at least version 0.1.15
    args = ['samtools']
    proc = subprocess.Popen(args, shell=False, cwd=os.getcwd(), stderr=subprocess.PIPE)
    for line in proc.stderr:
        c = line.split()
        if len(c) < 3:
            continue        
        if "Version:" in c[0]:
            major, minor, micro = [int(x) for x in c[1].split('.')]
            assert(minor >= 1)
            assert(micro >= 15)
    
    # Check that picard environment variable is set
    assert(os.environ['PICARD_DIR'])

    # Check for blat
    args = ['which', 'blat']
    proc = subprocess.Popen(args, shell=False, cwd=os.getcwd(), stderr=subprocess.PIPE)
    for line in proc.stderr:
        print line
        if "which: no blat in" in line:
            print "blat not found"
            sys.exit()

    return


if __name__ == '__main__':
    check_prerequisites()

