#!/usr/bin/env python
# encoding: utf-8
"""
interpolate_missing_bwasw_records.py

Created by Rod Docking on 2010-10-07.
Copyright (c) 2010 Canada's Michael Smith Genome Sciences Centre. All rights reserved.
"""

import sys
import os
import gzip

def read_fq_record(fq_handle):
    """Read the next fastq entry"""
    
    qname = fq_handle.readline().lstrip("@").rstrip()
    seq = fq_handle.readline().rstrip()
    fq_handle.readline()
    qual = fq_handle.readline().rstrip()    
    
    return qname, seq, qual
    
def main():
    """From a SAM file generated by BWASW, and the corresponding fastq file
    used to generate it, interpolate the missing records"""
    
    sam_handle = open(sys.argv[1],'r')
    fq_handle = open(sys.argv[2],'r')
    out_handle = open(sys.argv[3], 'w')
    
    prev_qname, prev_line = '', ''
    mid_fq_counter, end_fq_counter = 0, 0
    
    for line in sam_handle:
        
        # Write header lines
        if "@SQ" in line:
            out_handle.write(line)
            continue
        
        qname = line.split()[0]
        
        # Only write one record in cases where two are reported
        if qname == prev_qname:
            continue
        prev_qname = qname
        prev_line = line    
        
        # Check if the next record in the fastq file matches the
        #  next record from the SAM file.  If not, write
        #  fastq records until they're back in sync    
        # fq_qname, fq_seq, fq_qual = read_fq_record(fq_handle)    
        # while (not fq_qname == qname):
        #     out_handle.write("{0}\t4\t*\t0\t0\t*\t*\t0\t0\t{1}\t{2}\n".format(
        #         fq_qname, fq_seq, fq_qual))
        #     fq_qname, fq_seq, fq_qual = read_fq_record(fq_handle)
        #     mid_fq_counter += 1    

        out_handle.write(line)
    
    sam_handle.close()
    print "Added {0} fq records in the middle of the file".format(mid_fq_counter)
    
    # In case there are any unaligned reads at the end of the file
    lines = fq_handle.readlines()
    print "Found {0} remaining lines".format(len(lines))

    # counter = 0
    # for line in lines:
    #     counter += 1
    #     line = line.rstrip()
    #     
    #     if ((counter % 4) == 1):
    #         fq_qname = line.lstrip("@")
    #     elif ((counter % 4) == 2):     
    #         fq_seq = line
    #     elif ((counter % 4) == 0):    
    #         fq_qual = line
    #         out_handle.write("{0}\t4\t*\t0\t0\t*\t*\t0\t0\t{1}\t{2}\n".format(
    #             fq_qname, fq_seq, fq_qual))
    #         end_fq_counter += 1
    #                 
    # fq_handle.close()
    # print "And {0} fq records at the end".format(end_fq_counter)
    #     


if __name__ == '__main__':
    main()

